// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005-2007
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

// This is a library to access any bookmark service throw the Delicious API.
// Many services are providing a compatibility layer with Delicious,
// so this library is used also for Magnolia and Shadows.

// Date utility functions
var DateUtility = {
  // seconds since epoch
  secondsSinceEpoch:
      function secondsSinceEpoch () { return (new Date()).getTime()/1000; },

  re_iso8601: new RegExp(
      "^(\\d{4})(-(\\d{2})(-(\\d{2})" +
      "(T(\\d{2}):(\\d{2})(:(\\d{2})(\\.(\\d+))?)?" +
      "(Z|(([-+])(\\d{2}):(\\d{2})))?)?)?)?$"),
  parse: function parse(datestr) {
    var date = null;
    datestr = datestr.replace(/^\s*|\s*$/g, '');
    var dateval = Date.parse(datestr);

    if (dateval) {
      date = new Date(dateval);
    } else {
      /* this is a parser for the W3C subset of ISO 8601 dates
       * http://www.w3.org/TR/NOTE-datetime
       */
      var m = datestr.match(this.re_iso8601);

      if (m) {
        date = new Date(Date.UTC(m[1]))

        m[3]  && date.setUTCMonth   (m[3] - 1);
        m[5]  && date.setUTCDate    (m[5]);
        m[7]  && date.setUTCHours   (m[7]);
        m[8]  && date.setUTCMinutes (m[8]);
        m[10] && date.setUTCSeconds (m[10]);

        m[12] && date.setUTCMilliseconds(('0.' + m[12]) * 1000);

        if (m[14]) {
          var tz_off = m[16] * 60 + Number(m[17]);
          tz_off *= m[15] == '-' ? -1 : 1;
          tz_off *= 60 * 1000;

          date.setTime(date.getTime() - tz_off);
        }
      }
    }
    return date;
  },
  toIso: function DateUtility_toIso (d) {
    function zeropad (s, l) {
      s = s.toString(); // force it to a string
      while (s.length < l) {
        s = '0' + s;
      }
      return s;
    }

    // if d is a number, turn it into a date
    if (typeof d == 'number') {
      var dt = new Date()
      dt.setTime (d);
      d = dt;
    }

    // YYYY-MM-DDThh:mm:ssZ
    return zeropad (d.getUTCFullYear (), 4) + '-' +
           zeropad (d.getUTCMonth () + 1, 2) + '-' +
           zeropad (d.getUTCDate (), 2) + 'T' +
           zeropad (d.getUTCHours (), 2) + ':' +
           zeropad (d.getUTCMinutes (), 2) + ':' +
           zeropad (d.getUTCSeconds (), 2) + 'Z'
  }
}


function DeliciousAPI(aUrl) {
  this.url = aUrl;
}

DeliciousAPI.prototype = {
  call: function (aMethod, aArgs, aCallback, aAuth) {
    var argsarray = new Array ();
    for (var key in aArgs) {
      argsarray.push (encodeURIComponent (key) + '=' +
          encodeURIComponent (aArgs[key]))
    }

    var url = this.url + aMethod;

    if (argsarray.length) {
      url = url + '?' +argsarray.join ('&');
    }

    var request = Components.classes['@mozilla.org/xmlextras/xmlhttprequest;1'].createInstance(Components.interfaces.nsIXMLHttpRequest);

    var onReadyStateFunc = function(eEvt) {
      if (request.readyState == 4) {
        if (request.status >= 200 && request.status < 300) {
          var text = request.responseText.replace (/[\x00\x01\x02\x03\x04\x05\x06\x07\x08\x0B\x0C\x0E\x0F\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1A\x1B\x1C\x1D\x1E\x1F]/g, '');
          var xml = Cc['@mozilla.org/xmlextras/domparser;1']
              .createInstance(Ci.nsIDOMParser)
              .parseFromString (text, 'text/xml');
          aCallback.onSuccess (xml);
        } else {
          var error = Components.classes['@flock.com/error;1']
            .createInstance(Components.interfaces.flockIError);
          error.serviceErrorCode = request.status;
          error.serviceErrorString = request.responseText;
          switch (request.status) {
            case 503: // Service unavailable
              error.errorCode = error.FAVS_UNAVAILABLE;
              break;
            case 401: // Bad login/password
              error.errorCode = error.FAVS_INVALID_AUTH;
              break;
            default: // Unknown error code
              error.errorCode = error.FAVS_UNKNOWN_ERROR;
          }
          aCallback.onError (error);
        }
      }
    };

    request.onreadystatechange = onReadyStateFunc;
    request.detachLoadGroup = true;
    request.overrideMimeType("text/txt");
    if (aAuth)
      request.open('GET', url, true, aAuth.user, aAuth.password);
    else
      request.open('GET', url, true);
    request.send(null);
  },

  // Make sure to call posts/update before calling this
  postsAll: function (aListener, aAuth, aSeparator) {
    if (!aSeparator) aSeparator = ' ';
    this.call ('posts/all', {}, {
      onError: function (aError) {
        aListener.onError (aError);
      },
      onSuccess: function (aXML) {
        if (!aXML || !aXML.documentElement ||
          aXML.documentElement.tagName != 'posts') {
          aListener.onError ();
          return;
        }
        var result = [];
        var children = aXML.documentElement.childNodes;
        for (var i=0; i<children.length; i++) {
          var child = children.item (i);
          if (child.nodeType != 1) {
            continue;
          }
          child.QueryInterface (Ci.nsIDOMElement);
          if (child.tagName != 'post') {
            continue;
          }
          if ((child.getAttribute('href') == 'about:blank') ||
              (child.getAttribute('href') == ''))
            continue;
          result.push({
            URL: child.getAttribute('href'),
            name: child.getAttribute ('description'),
            description: child.getAttribute ('extended'),
            tags: child.getAttribute('tag'),
            tag: child.getAttribute('tag').split(aSeparator),
            shared: child.getAttribute('shared'),
            datevalue: DateUtility.parse(child.getAttribute('time')),
            hash: child.getAttribute('hash')
          });
        }
       
        aListener.onSuccess (result, "updated");
      }
    }, aAuth);
  },

  // Returns null if there is no new stuff on the server, all the posts otherwise
  postsAllIfNewer: function (aListener, aAuth, aLastUpdate, aSeparator) {
    if (aLastUpdate.getTime() == 0) {
      // First refresh: let's go ahead and grab the posts
      this.postsAll(aListener, aAuth, aSeparator);
    }
    else {
      var api = this;
      this.call ('posts/update', {}, {
        onError: function (aError) {
          // FIXME: report error?
          /* BUG: 5705 */
          if (aListener)
            aListener.onError (aError);
        },
        onSuccess: function (aXML) {
          if (!aXML || !aXML.documentElement ||
              aXML.documentElement.tagName != 'update') {
            // bad posts/all response
            // FIXME: report error?
            /* BUG: 5705 */
            if (aListener)
              aListener.onError ();
            return;
          }
          var lastUpdate = DateUtility.parse(aXML.documentElement.getAttribute("time"));
          //svc._logger.debug ("Last update on the server: "+lastUpdate);
          //svc._logger.debug ("Last time we got stuff from the server: "+aLastUpdate);
          if (lastUpdate.getTime() > aLastUpdate.getTime()) {
            // There is new/updated stuff on the server
            if (api.timer)
              api.timer.cancel();
            else
              api.timer = Components.classes['@mozilla.org/timer;1'].createInstance(Components.interfaces.nsITimer);
            api.timer.initWithCallback({
              notify: function(aTimer) { api.postsAll(aListener, aAuth, aSeparator); }
            }, 1300, Ci.nsITimer.TYPE_ONE_SHOT);
          }
          else {
            // No refresh needed
            if (aListener)
              aListener.onSuccess(null, "nonew");
          }
        }
      }, aAuth);
    }
  },

  tagsGet: function (aListener, aAuth) {
    this.call ('tags/get', {}, {
      onError: function () {
        if (aListener)
          aListener.onError ();
      },
      onSuccess: function (aXML) {
        if (!aXML || !aXML.documentElement ||
          aXML.documentElement.tagName != 'tags') {
          aListener.onError ();
          return;
        }
        var result = [];
        var children = aXML.documentElement.childNodes;
        for (var i=0; i<children.length; i++) {
          var child = children.item (i);
          if (child.nodeType != 1) continue;
          child.QueryInterface (Ci.nsIDOMElement);
          if (child.tagName != 'tag') continue;
          result.push({
            tag: child.getAttribute('tag'),
            count: child.getAttribute('count')
          });
        }
        aListener.onSuccess(result, "success");
      }
    }, aAuth);
  }

}


